;; Subtitler - sync transcripts into mpsub files

;; https://www.fsfla.org/blogs/lxo/2021-02-28-syncing-subtitles-in-freedom

;; Bind some key, say RET, to subtitler-insert-time, then type it at
;; the time and point you wish to end a subtitle and start another.

;; Copyright 2021 Alexandre Oliva <oliva@gnu.org>

;; This file is an extension for GNU Emacs.

;; This file is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this file.  If not, see <http://www.gnu.org/licenses/>.

(provide 'subtitler)

(defvar subtitler-base-time nil
  "This is the start time for the currently-open subtitle entry.
Set it to (current-time) to resync.")

(defvar subtitler-auto-end-of-line nil
  "Set this to t to move to the end of the line after starting a new subtitle entry.  This is useful if the transcript already has line breaks at suitable places.")

(defun subtitler-insert-time ()
  "Insert an mpsub subtitle break at the current point.
Bind this interactive function to RET to conveniently add timing
info to a transcript.

If it's the first subtitle entry, the file header is inserted,
and time-keeping is started.

Otherwise, the currently open subtitle entry is closed with the
duration since it was started, and a new one is opened with a
zero-length gap.

Use blank lines for no-text intervals; they can be cleaned up
mechanically later.
" (interactive)
  (let ((next-time (current-time)))
    (insert "\n\n0 ")
    (if (not (looking-at "\n"))
	(insert "\n")
      (goto-char (+ (point) 1)))
    (if (not (search-backward "\n\n0 " nil t 2))
	(progn (goto-char (point-min))
	       (insert "FORMAT=TIME"))
      (goto-char (match-end 0))
      (insert (format "%.1f"
		      (float-time (time-subtract next-time
						 subtitler-base-time)))))
    (search-forward "\n\n0 \n")
    (goto-char (match-end 0))
    (if (subtitler-auto-end-of-line)
	(end-of-line))
    (setq subtitler-base-time next-time)))
